@@ -1,6 +1,7 @@
 """Configuration.
 """
-from os import path
+import os.path
+import tempfile
 
 #: what to download
 VEKN_TWDA_URL = "http://www.vekn.fr/decks/twd.htm"
@@ -9,11 +10,9 @@ VEKN_VTES_LIBRARY_FILENAME = "vteslib.csv"
 VEKN_VTES_CRYPT_FILENAME = "vtescrypt.csv"
 VEKN_VTES_ENCODING = "utf-8"
 #: where to store our data
-TWDA_FILE = path.join(path.dirname(__file__), "TWDA.pkl")
-VTES_FILE = path.join(path.dirname(__file__), "VTES.pkl")
-CARDS_RULINGS_FILE = path.join(path.dirname(__file__), "cards-rulings.yaml")
-GENERAL_RULINGS_FILE = path.join(path.dirname(__file__), "general-rulings.yaml")
-RULINGS_LINK_FILE = path.join(path.dirname(__file__), "rulings-links.yaml")
+TEMP_DIR = tempfile.gettempdir()
+TWDA_FILE = os.path.join(TEMP_DIR, "krcg-twda.pkl")
+VTES_FILE = os.path.join(TEMP_DIR, "krcg-vtes.pkl")
 #: classic headers in deck lists
 HEADERS = [
     "master",
@@ -60,8 +59,6 @@ ADDITIONAL_HEADERS = [
 AKA = {
     # print change not registered in cards "Aka" field
     "mask of 1,000 faces": "Mask of a Thousand Faces",
-    # typo in VEKN official file
-    "thadius zho": "Thadius Zho{}",
 }
 
 CLANS_AKA = {
@@ -147,6 +144,97 @@ DISC_AKA = {
     "vis": "Visceratika",
 }
 
+#: registered traits
+TRAITS = [
+    "primogen",
+    "prince",
+    "justicar",
+    "inner Circle",
+    "imperator",
+    "bishop",
+    "archbishop",
+    "cardinal",
+    "regent",
+    "priscus",
+    "baron",
+    "liaison",
+    "magaji",
+    "kholo",
+    "black hand",
+    "seraph",
+    "red list",
+    "infernal",
+    "slave",
+]
+
+#: disciplines trigrams used in crypt CSV
+DIS_MAP = {
+    "-none-": "none",
+    "abo": "abombwe",
+    "ani": "animalism",
+    "aus": "auspex",
+    "cel": "celerity",
+    "chi": "chimerstry",
+    "dai": "daimoinon",
+    "def": "defense",
+    "dem": "dementation",
+    "dom": "dominate",
+    "for": "fortitude",
+    "inn": "innocence",
+    "jud": "judgment",
+    "mar": "martyrdom",
+    "mel": "melpominee",
+    "myt": "mytherceria",
+    "nec": "necromancy",
+    "obe": "obeah",
+    "obf": "obfuscate",
+    "obt": "obtenebration",
+    "pot": "potence",
+    "pre": "presence",
+    "pro": "protean",
+    "qui": "quietus",
+    "red": "redemption",
+    "san": "sanguinus",
+    "ser": "serpentis",
+    "spi": "spiritus",
+    "tem": "temporis",
+    "tha": "thaumaturgy",
+    "thn": "thanatosis",
+    "val": "valeren",
+    "ven": "vengeance",
+    "vic": "vicissitude",
+    "vii": "vision",  # VIS is used in CSV
+    "vis": "visceratika",
+    "abo": "abombwe",
+    "ANI": "ANIMALISM",
+    "AUS": "AUSPEX",
+    "CEL": "CELERITY",
+    "CHI": "CHIMERSTRY",
+    "DAI": "DAIMOINON",
+    "DEM": "DEMENTATION",
+    "DOM": "DOMINATE",
+    "FOR": "FORTITUDE",
+    "MEL": "MELPOMINEE",
+    "MYT": "MYTHERCERIA",
+    "NEC": "NECROMANCY",
+    "OBE": "OBEAH",
+    "OBF": "OBFUSCATE",
+    "OBT": "OBTENEBRATION",
+    "POT": "POTENCE",
+    "PRE": "PRESENCE",
+    "PRO": "PROTEAN",
+    "QUI": "QUIETUS",
+    "SAN": "SANGUINUS",
+    "SER": "SERPENTIS",
+    "SPI": "SPIRITUS",
+    "TEM": "TEMPORIS",
+    "THA": "THAUMATURGY",
+    "THN": "THANATOSIS",
+    "VAL": "VALEREN",
+    "VIC": "VICISSITUDE",
+    "VIS": "VISCERATIKA",
+}
+
 #: custom remap to match players abbreviations and typos
 REMAP = {
     # our card parsing removes the dash
@@ -347,6 +435,15 @@ REMAP = {
     "soul gems": "Soul Gem of Etrius",
     "tomb of rameses the cheesemonger": "Tomb of Rameses III",
     "truth of a 1000 lies": "Truth of a Thousand Lies",
+    # 🥚
+    "enkil cock": "enkil cog",
+    "parity shit": "Parity Shift",
+    "heart of the shitty": "Heart of the City",
+    "heart of cheating": "Heart of Nizchetus",
+    "bounce": "Deflection",
+    "no": "Direct Intervention",
+    "bye": "Banishment",
+    "hero fellation": "Aire of Elation",
 }
 #: type order for deck display
 TYPE_ORDER = [
