@@ -7,6 +7,7 @@ import collections
 import io
 import itertools
 import logging
+import os
 import pickle
 import re
 
@@ -43,7 +44,7 @@ class _TWDA(collections.OrderedDict):
     """ An OrderedDict of the TWDA. Parsing TWDA.html is the hard part.
 
     Attributes:
-        spoilers (set): cards played in over 25% of decks
+        spoilers (dict): cards played in over 25% of decks
         tail_re (str): regexp used to parse tail part of card line
     """
 
@@ -54,6 +55,7 @@ class _TWDA(collections.OrderedDict):
             limit: Maximum number of decks to load (used to speed up tests)
         """
         r = requests.request("GET", config.VEKN_TWDA_URL)
+        r.raise_for_status()
         self.load_html(io.StringIO(r.content.decode("utf-8")), limit, save)
 
     def load_html(self, source, limit=None, save=True):
@@ -74,7 +76,7 @@ class _TWDA(collections.OrderedDict):
                 for s in vtes.VTES.clans + vtes.VTES.disciplines + ["trifle"]
             )
         )
-        for count, (lines, twda_id) in enumerate(self._get_decks_html(source)):
+        for count, (lines, twda_id) in enumerate(self._get_decks_html(source), 1):
             try:
                 self[twda_id] = self._load_deck_html(lines, twda_id)
             except arrow.parser.ParserError as e:
@@ -85,7 +87,7 @@ class _TWDA(collections.OrderedDict):
         if save:
             pickle.dump(TWDA, open(config.TWDA_FILE, "wb"))
 
-    def configure(self, date_from=None, date_to=None, min_players=0):
+    def configure(self, date_from=None, date_to=None, min_players=0, spoilers=True):
         """Prepare the TWDA, taking date and players count filters into account.
 
         Also compute `self.spoilers`.
@@ -94,6 +96,7 @@ class _TWDA(collections.OrderedDict):
             date_from (datetime): filter out decks before this date
             date_to (datetime): filter out decks after this date
             min_players (int): if > 0, filter out decks with less players
+            spoilers (bool): if True, compute spoilers to filter them out for analysis
         """
         if date_from:
             for key in [key for key, value in self.items() if value.date < date_from]:
@@ -108,9 +111,9 @@ class _TWDA(collections.OrderedDict):
                 if value.players_count and value.players_count < min_players
             ]:
                 del self[key]
-        if len(self) > 50:
+        if spoilers and len(self) > 50:
             self.spoilers = {
-                name
+                name: count / len(self)
                 for name, count in collections.Counter(
                     itertools.chain.from_iterable(d.keys() for d in self.values())
                 ).items()
@@ -428,6 +431,8 @@ def _get_card(line):
 
 
 try:
+    if not os.path.exists(config.TWDA_FILE) or os.stat(config.TWDA_FILE).st_size == 0:
+        raise FileNotFoundError(config.TWDA_FILE)
     TWDA = pickle.load(open(config.TWDA_FILE, "rb"))
 except (FileNotFoundError, EOFError):
     TWDA = _TWDA()  # evaluates to False as it is empty
